importPackage(java.lang);
importPackage(java.sql);
importPackage(java.util);
importPackage(java.io);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.alarmpoint.integrationagent.util.EventDeduplicator);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);

var xStream = new XStream(new PureJavaReflectionProvider())

// Core JS files
load("integrationservices/ibmnetcool/baseclass.js");
load("integrationservices/ibmnetcool/logger.js");
load("integrationservices/ibmnetcool/wsutil.js");
load("integrationservices/ibmnetcool/xmattersws.js");


// Integration-specific JS files
load("integrationservices/ibmnetcool/ibmnetcool-config.js");

var exceptionString = "JavaException: ";
var notePrefix = "[xMatters] - "; // The Prefix which is added to the note comments e.g. "[xMatters] "
var calloutUser = "xMatters"; // Username associated with callout annotations


//-----------------------------------
// Shared globals below this line
//-----------------------------------

var log = new Logger("ibmnetcool.js: ");

//---------------------------------------------------------------------------------------
// Functions and globals below this line are used by the direct xMatters INTEGRATION
//---------------------------------------------------------------------------------------

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by APClient.  The messages may be requests to perform
 * local activity, or they may be requests to make submissions to xMatters.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to xMatters
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml)
{
  var result = null;
  
  // Change the severity to be readable
  if (apxml.getToken("severity") != null) 
  {
    var severity = apxml.getToken("severity").getValue();
    log.debug(" Severity enrichment for sev:" + severity);
	
    //Create a displayable severity variable
	if ( severity == "5" ) 
	  apxml.setToken("severity", "Critical");
	else if ( severity == "4" ) 
	  apxml.setToken("severity", "Major");
	else if ( severity == "3" ) 
	  apxml.setToken("severity", "Minor");
	else if ( severity == "2" ) 
	  apxml.setToken("severity", "Warning");
	else if ( severity == "1" ) 
	  apxml.setToken("severity", "Indeterminate");
	else if ( severity == "0" ) 
	  apxml.setToken("severity", "Clear");
  }

  log.debug(" APXML to be sent to xMatters: " + apxml);

  // Forward enriched APXML to AlarmPoint, subject to deduplication
  var webService = new xMattersWS();
  var recentlyReceived = EventDeduplicator.getInstance().recentOccurrenceCount(apxml, DEDUPLICATOR_FILTER);

  if (recentlyReceived <= 1)
  {
      result = apxml;
  }
  else
  {
      log.warn("***Deduplicator Suppressed Notification ***: An event with tokens " + webService.convertTokensToString(apxml) + " has been injected into the " +
          apxml.getValue("agent_client_id") + " event domain " + recentlyReceived + " times within the configured suppression period.  It has been suppressed." );
  }

  return result;
}

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by xMatters.  The messages may be responses to
 * previous submissions, or they may be requests to the Integration Service
 * that originate from xMatters
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to xMatters
 * either via the Integration Service's outbound queues or directly as a
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml)
{
  var method = apxml.getMethod();
  log.debug("************ APIA_RESPONSE method = " + method);
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  }
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  }
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}

/**
 * OK responses are returned by xMatters for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * This implementation just logs the ack.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  log.info("Submission of transaction " + id + " succeeded.");
}

/**
 * ERROR responses are returned by xMatters for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * This implementation just logs the error.
 */
function handleERROR(apxml)
{
  var transaction = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE);
  log.error("Submission of transaction " + transaction + " failed with error code " + code);
}

/**
 * Requests are forwarded to the Send handler.
 * <p>
 * Redirects to the handleSend as messages may come from ESM or ESR to the IA.
 */
function handleRequest(apxml)
{
    log.debug( "handleRequest received apxml, forwarding to handleSend" );
    return handleSend(apxml);
}

/**
 * The main routine for handling Requests starts here.
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml)
{

  log.debug( "Enter - handleSend received apxml: " + apxml );
  var result = null;

  // The presence of the message token means this message is a callout annotation.
  var message = apxml.getValue("message");
  var incidentId = apxml.getToken("incident_id").getValue();

  if ( message != null && incidentId != null)
  {
      result = handleCalloutAnnotate(incidentId, message, apxml);
  }
  else
  {
      result = handleApsResponses( apxml );
  }

  log.debug( "Exit - handleSend");
  return result;
}


/**
 * -----------------------------------------------------------------------------
 * handleApsResponses
 *
 * Handling of delivery notifications and user responses from xMatters.
 * Decide what to do with the APXML message that has been received.
 * -----------------------------------------------------------------------------
 */
function handleApsResponses(apxml)
{
    log.debug( "Enter - handleApsResponses");

    // Initialize xMattersWS interface building a reply to xMatters.
    var xmWebService = new xMattersWS();
    var responseToXM = xmWebService.initializeResponse(apxml);
    var responseTokens = getTokens(apxml);

    try
    {
        log.debug( "handleApsResponses - Token keys from xMatters: " + responseTokens);

        var responseAction = responseTokens.get("responseAction");
        var annotation = responseTokens.get("annotation");

        // See if the APXML message is invalid, or if we don't have anything to do
        // in response to it.
        if (!checkAPXMLTokens(responseTokens))
        {
            // Nothing to do in response to this message, so just return OK
            responseToXM.setToken(xmWebService.TOKEN_NAME, "OK", APXMLToken.Type.STRING);
        }
        else
        {
            if (responseTokens.get("responseEvent") == "SUCCESSFUL_DELIVERY"
                || responseTokens.get("responseEvent") == "DELIVERY_FAILURE")
            {
                log.debug("Received delivery notification [" + responseTokens.get("responseEvent") + "]");
                // Create an annotation for the successful or failed delivery of a notification.
                // Set the responseAction token to Annotate to pass control to the Annotate handler.
                responseAction = "Annotate";
                annotation = responseTokens.get("responseEvent") + " to " + responseTokens.get("responder");
            }

            handleResponseAction(responseAction, annotation, responseTokens, responseToXM);

            responseToXM.setToken(xmWebService.TOKEN_NAME, "OK", APXMLToken.Type.STRING);
        }
    }
    catch (e)
    {
        var errMsg = "Caught exception processing [" + responseTokens.get("responseEvent") + "|" + responseAction + "]: " + e;
        log.error(errMsg);
        responseToXM.setToken(xmWebService.TOKEN_NAME_EXCEPTION, errMsg.replace(exceptionString, ""), APXMLToken.Type.STRING);
    }
    
    log.debug( "Exit - handleApsResponses");
    return responseToXM;
}

/**
 * -----------------------------------------------------------------------------
 * handleResponseAction
 *
 * Dispatch the user response to the appropriate handler function. The caller
 * may have over-ridden the responseAction and annotation tokens from the APXML
 * message, but the other tokens will be untouched.
 * -----------------------------------------------------------------------------
 */
function handleResponseAction(responseAction, annotation, responseTokens, responseToXM)
{
    log.debug("Enter - handleResponseAction. Handling responseAction [" + responseAction + "] for event [" + responseTokens.get("incidentId") + "]");

    var incidentId = responseTokens.get("incidentId");
    var responder = responseTokens.get("responder");
    var responderId = responseTokens.get("responderId");
    var sendAnnotations = "true".equalsIgnoreCase(responseTokens.get("sendAnnotations"));

    var updateStatus = null;
    var journalMessage = null;

    var serverName = responseTokens.get("servername");
    var serverSerial = responseTokens.get("serverserial");
    
    if (annotation != null)
    {
        annotation = notePrefix + annotation;
    }

    switch (String(responseAction).toLowerCase())
    {
        case "acknowledge":
            updateStatus =
              "Update alerts.status set XMState = 4, Acknowledged=true, XMResponder = '" +
              escapeSqlChars( responder ) +
              "', XMLastUpdated = getdate where ServerName = '" + serverName +
              "' and ServerSerial = " + serverSerial + ";";

            journalMessage = responder + " has acknowledged the event.";
            journalMessage = escapeSqlChars( journalMessage );

            logStatus( "xMatters event has been acknowledged.",
              updateStatus,
              journalMessage );
            break;
        case "own":

            // get the owner id based off the responder name
            var sql = "Select UID from master.names where Name = '" + responderId + "';";
            var uid = select(sql, "UID");
            
            if (uid != null && uid != "") {
              updateStatus =
                "Update alerts.status set XMState = 9, OwnerUID=" + uid + ", XMResponder = '" +
                escapeSqlChars( responder ) +
                "', XMLastUpdated = getdate where ServerName = '" + serverName +
                "' and ServerSerial = " + serverSerial + ";";

              journalMessage = responder + " has owned the event.";
              journalMessage = escapeSqlChars( journalMessage );

              logStatus( "xMatters event has been owned.",
                updateStatus,
                journalMessage );
            } else 
              throw responderId + " unable to be found in Netcool therefor the OWN response will not work";

            break;

        case "ignore":
            updateStatus =
              "Update alerts.status set XMState = 8, XMResponder = '" +
              escapeSqlChars( responder ) + "', " +
              "XMLastUpdated = getdate where ServerName='" + serverName +
              "' and ServerSerial = " + serverSerial + " and XMState != 4;";

            journalMessage = responder + " is ignoring this problem.";
            journalMessage = escapeSqlChars( journalMessage );

            logStatus( "Ignoring the problem.",
              updateStatus,
              journalMessage );
            break;

        case "annotate":
            annotation = annotation + " from " + responder;
            journalMessage = escapeSqlChars( annotation );

            updateStatus =
              "Update alerts.status set " +
              "XMLastUpdated = getdate where ServerName='" + serverName +
              "' and ServerSerial = " + serverSerial + ";";
            break;
        default:
            throw "Unknown response Action: " + responseAction;
            break;
    }

    // Only if a valid request was selected should an attempt be made to run SQL.
    //
    if( updateStatus != null && serverSerial != null )
    {
       update( updateStatus, journalMessage, serverSerial );
    }

    log.debug("Exit - handleResponseAction");
}

/*
 * checkAPXMLTokens
 *
 * Check that the received response tokens are valid and see if this message needs to be acted on. 
 * Throws an exception if message is invalid.
 * Returns true if the message needs to be processed.
 */
function checkAPXMLTokens(responseTokens)
{
    var result = true;

    switch (String(responseTokens.get("responseEvent")))
    {
        case "RECEIVED_RESPONSE":
        case "STANDALONE_ANNOTATION":
            if(responseTokens.get("responseAction") == null
                || responseTokens.get("responseAction") == ""
                || responseTokens.get("responseAction") == undefined)
                {
                throw "responseAction not set for a RECEIVED_RESPONSE responseEvent.  responseAction required check xMatters scripting to ensure this token is set";
            }
            break;

        case "DEVICE_DELIVERED":
            // Log voice device delivery, but do not annotate to the MS, a callout annotation will be used instead.
            // So just return.
            log.debug(notePrefix + responseTokens.get("responseEvent") + " to " + responseTokens.get("responder"));
            result = false;
            break;

        case "SUCCESSFUL_DELIVERY":
        case "DELIVERY_FAILURE":
            if (!ANNOTATE_DELIVERY)
            {
                log.warn("DELIVERY_ANNOTATIONS are suppressed the following message will not be annotated to IBMNETCOOL event: ["
                    + responseTokens.get("incidentId") + "] " + responseTokens.get("annotation"));
                result = false;
            }
            break;

        default:
            throw "Unknown responseEvent.  Expected: RECEIVED_RESPONSE||SUCCESSFUL_DELIVERY||DELIVERY_FAILURE||DEVICE_DELIVERED. Received: " 
            + responseTokens.get("responseEvent");
            break;
    }
   
    return result;
}

// -----------------------------------------------------------------------------
// Callout annotation processing
//
// If enabled, these result in annotations (notes) being added to the event.
// -----------------------------------------------------------------------------
function handleCalloutAnnotate(incidentId, messageText, apxml)
{

    log.debug("Enter - handleCalloutAnnotate - incidentId [" + incidentId + "], messageText [" + messageText + "]");

    if(!ANNOTATE_DELIVERY)
    {
        log.warn("DELIVERY_ANNOTATIONS are suppressed the following message will not be annotated to IBMNETCOOL event: [" + incidentId + "] " + messageText);
        return;
    }

    log.debug( "Starting call to management system for callout annotate for [" + incidentId + "]" );

    try
    {
      log.debug("Finished call to management system for callout annotate for [" + incidentId + "]");
    }
    catch (e)
    {
        log.error("Caught exception processing callout annotate for [" + incidentId + "] Exception:" + e);
        throw e;
    }

    log.debug("Exit - handleCalloutAnnotate");
}

// -----------------------------------------------------------------------------
// Utility functions
// -----------------------------------------------------------------------------

/**
 * Retrieves the additional tokens from the APXML
 * @param apxml XML message to process
 * @return additional tokens
 * @throws if the additional tokens are missing from the XML.
 */
function getTokens(apxml)
{
    if (apxml.getValue("additionaltokens") != null && !"".equalsIgnoreCase(apxml.getValue("additionaltokens")))
    {
        return xStream.fromXML(apxml.getValue("additionaltokens")); // LinkedHashMap
    }
    throw new Exception("additionaltokens is missing from apxml");
}

/** ***********************************************************************
 *
 * DESCRIPTION
 * This procedure will escape ' and \ characters to prevent conflicts with the
 # SQL command.
 *
 * PARAMETERS
 * sqlChars - The string to escape.
 *
 * RETURNS
 * The escaped sqlChars string.
 *
 ** ***********************************************************************/
function escapeSqlChars( sqlChars )
{
  sqlChars = sqlChars.replace("\\", "\\\\");
  return sqlChars.replace("'", "\\'");
}

/** ***********************************************************************
 *
 * DESCRIPTION
 * This procedure will preform a SQL Select from the OMNI database.
 *
 * PARAMETERS
 * sql - The sql to execute within Sybase.
 * column - The column to return
 *
 * RETURNS
 * ResultSet of the selected statement.
 *
 ** ***********************************************************************/
function select( sql, column )
{

  var result;

  try 
  {
    Class.forName( "com.sybase.jdbc2.jdbc.SybDriver" );

    var con = DriverManager.getConnection(
      OMNI_os_constr,
      OMNI_NetcoolUser,
      OMNI_netcool_password );

    log.debug(" SQL to be executed: " + sql );
    var stmt = con.createStatement();
    var rs = stmt.executeQuery( sql );
    while (rs.next())
    {
      result = rs.getString(column);
      log.debug(" result: " + result );
    }

  
    con.close();
  }
  catch( error )
  {
    AGENT_LOG_MESSAGE = "A problem has occurred : " + error;
    log.error( AGENT_LOG_MESSAGE );
  }

  return result;
}

/** ***********************************************************************
 *
 * DESCRIPTION
 * This procedure will execute sql, which updates the OMNI database with
 * data.
 *
 * PARAMETERS
 * sql - The sql to execute within Sybase.
 * sqljournal - A message noting why the SQL was performed.
 *
 * RETURNS
 * The number of rows updated or inserted.
 *
 ** ***********************************************************************/
function update( sql, sqljournal, serverSerial )
{
  var res_s = 0;
  var res_j = 0;
  
  sqljournal = annotationPrefix + sqljournal;

  var dt = new Date();
  var otime = dt.getTime();

  var OMNI_KeyField = serverSerial + ":" + OMNI_UID + ":" + otime;

  var sqlj =
    "insert into alerts.journal (KeyField,Serial,UID,Chrono,Text1) values ( '"
    + OMNI_KeyField + "'," + serverSerial + "," + OMNI_UID + ",getdate,'"
    + sqljournal + "');";

  try 
  {
    Class.forName( "com.sybase.jdbc2.jdbc.SybDriver" );

    var con = DriverManager.getConnection(
      OMNI_os_constr,
      OMNI_NetcoolUser,
      OMNI_netcool_password );

    log.debug(" SQL to be executed: " + sql );
    var stmt = con.createStatement();
    res_s = stmt.executeUpdate( sql );

    log.debug(" SQL to be executed: " + sqlj );
    res_j = stmt.executeUpdate( sqlj );
    
    con.close();
  }
  catch( error )
  {
    AGENT_LOG_MESSAGE = "A problem has occurred : " + error;
    log.error( AGENT_LOG_MESSAGE );
  }

  // We return the number of rows affected which should always
  // be 1. 0 would be some kind of SQL Error and -1 would be either
  // script error or database connection failed etc..
  //
  return res_s + res_j;
}

/** ***********************************************************************
 *
 * DESCRIPTION
 * This procedure will log the given parameters to disk, and the last two
 * parameters to standard output as well.
 *
 * OUTPUTS
 * The updateStatus and journalMessage to console
 * The msgStatus, updateStatus, and journalMessage to the log.
 *
 ** ***********************************************************************/
function logStatus( msgStatus, updateStatus, journalMessage )
{
  log.debug( updateStatus );
  log.debug( journalMessage );
  log.debug( msgStatus );
}
