importPackage(java.lang);
importPackage(java.util);
importPackage(java.io);
importClass(Packages.com.alarmpoint.integrationagent.config.IAConfigImpl);
importClass(Packages.com.alarmpoint.integrationagent.config.xml.IAConfigFileImpl);

/******************************************************************************/
/*        Start of xMatters Webservices Base Class Object Definition          */
/******************************************************************************/
var failedList = new ArrayList();
var successList = new ArrayList();

var xmns = new Namespace("http://www.alarmpoint.com/webservices/schema");

var xMattersWS = WSUtil.extend({

  XMATTERS_WSDL_DEFN: "integrationservices/bmcbppm/soap/AlarmPoint-soapui-project.xml",
  XM_SOAP_VERSION: "1_1",

  IA_CONFIG_FILE_PATH: "conf/IAConfig.xml",

  // xMatters web service responses
  SERVICE_DENIED: "SERVICE_DENIED",
  UNKNOWN_INCIDENT: "UNKNOWN_INCIDENT",

  TOKEN_NAME: "apiaResult",
  TOKEN_NAME_EXCEPTION: "apiaResultException",

  /**
   * Constructor
   */
  init: function()
  {
    this._super();

    var configFile = new File(this.IA_CONFIG_FILE_PATH);
    var iaConfigFile = new IAConfigFileImpl(new FileReader(configFile));
    var iaConfig = new IAConfigImpl(iaConfigFile, configFile.getParentFile(), configFile.toURI());

    this.wsUser = iaConfigFile.getWSUser();
    this.wsPassword = iaConfig.getWsAuth().getPassword();
    this.wsCompany = iaConfigFile.getWSCompany();
    this.endPoint = iaConfigFile.getPrimaryServerUrls().next();
  },

  /**
   * Sends the request to the SOAP endpoint and returns the response
   * @param url EndPoint for the webservice request
   * @param msg SOAPMessage instance containing the SOAPEnvelope to post
   * @param objectType the type of object being synchronized (User, Coverage, Device, Group, Team)
   * @param objectName the name of the object being synchronized
   * @return SOAP Response Body
   */
  syncSendReceive: function(url, msg, objectType, objectName)
  {
    var response = this.sendReceive(url, msg);

    default xml namespace = xmns;
    var status = response..status;

    if (!"OK".equalsIgnoreCase(status))
    {
      failedList.add(objectType + ": " + objectName + " failed to synchronize: " + status);
    }
    else
    {
      successList.add(objectType + ": " + objectName + " synchronized: " + status);
    }
    return response;
  },

  /**
   * Query xMatters for the incident
   * @param incidentId id of the incident to retrieve
   * @return true if the incident exists, false otherwise
   */
  queryIncident: function(incidentId)
  {
    var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "QueryIncident", "QueryIncident", this.XM_SOAP_VERSION);

    // Set the parameter values
    msg.setParameter("user", this.wsUser);
    msg.setParameter("password", this.wsPassword);
    msg.setParameter("company", this.wsCompany);
    msg.setParameter("incidentIdentifier", incidentId);

    // Post the message to the SOAP endpoint
    var response = this.sendReceive(this.endPoint, msg);

    var status = response.xmns::QueryIncidentResponse.child(QName(xmns, "return")).xmns::status;
    this.log.debug("Response Status: " + status);
    var exists = !this.UNKNOWN_INCIDENT.equalsIgnoreCase(status) && !this.SERVICE_DENIED.equalsIgnoreCase(status);

    return exists;
  },

  /**
   * Send a DEL APXML message if an Event with the given Incident ID exists in xMatters
   * @param incidentId id of the incident/event that will be terminated
   */
  sendDelAPXML: function(incidentId)
  {
    // Check if the notification exists in xMatters
    if (this.queryIncident(incidentId))
    {
      var apxml = ServiceAPI.createAPXML();
      apxml.setMethod("DEL");
      apxml.setSubclass("action");
      apxml.setToken(APXMLMessage.APIA_PROCESS_GROUP, incidentId);
      apxml.setToken("incident_id", incidentId);

      log.debug("Built DEL apxml: " + apxml);

      try
      {
        ServiceAPI.sendAPXML(apxml);
      }
      catch (e)
      {
        log.error("Caught exception sending del to xMatters for Incident ID: " + incidentId);
        throw e;
      }

      log.debug("DEL apxml sent to xMatters");
    }
  },

  /**
   * Submits the APXML to xMatters for processing
   * @param apxml APXML to submit
   */
  submitAPXML: function(apxml, delExistingEvents)
  {
    // Invoke event de-duplication for any other event
    // if we haven't got more than 1 event in the programmed timeout
    var recentlyReceived = EventDeduplicator.getInstance().recentOccurrenceCount(apxml, DEDUPLICATOR_FILTER);
    if (recentlyReceived <= 1)
    {
      if (delExistingEvents)
      {
        this.sendDelAPXML(apxml.getValue("incident_id"));
      }
      ServiceAPI.sendAPXML(apxml);
      log.debug("Finished sending apxml to xMatters");
    }
    else
    {
      log.warn("***Deduplicator Suppressed Notification ***: An event with tokens " + this.convertTokensToString(apxml) + " has been injected into the " +
               apxml.getValue("agent_client_id") + " event domain " + recentlyReceived + " times within the configured suppression period.  It has been suppressed." );
    }
  },

  /**
   * Initializes the response to be returned to the device
   * @param apxml APXML containing the request information
   * @return initialized instance of the response APXML
   */
  initializeResponse: function(apxml)
  {
    var response = ServiceAPI.createAPXML();
    response.setMethod(APXMLMessage.RESPONSE_METHOD);
    response.setSubclass(APXMLMessage.RESPONSE_SUBCLASS);

    if (apxml.hasToken(APXMLMessage.REQUEST_ID))
    {
      response.setToken(APXMLMessage.REQUEST_ID, apxml.getValue(APXMLMessage.REQUEST_ID), APXMLToken.Type.STRING);
    }
    response.setToken(APXMLMessage.DESTINATION, apxml.getValue(APXMLMessage.ORIGINATOR), APXMLToken.Type.STRING);
    if (apxml.hasToken(APXMLMessage.APIA_PRIORITY))
    {
      response.setToken(APXMLMessage.APIA_PRIORITY, apxml.getValue(APXMLMessage.APIA_PRIORITY));
    }
    else
    {
      response.setToken(APXMLMessage.APIA_PRIORITY, APXMLMessage.APIA_PRIORITY_HIGH);
    }
    return response;
  },

  /**
   * Convert all tokens for this message into a parenthesis-grouped set of name-value-pairs.
   * @param apxml tokens to be converted into a string
   * @return string representation of input tokens
   */
  convertTokensToString: function(apxml)
  {
    var string = "";
    var tokenIterator = apxml.getTokens();
    while (tokenIterator.hasNext())
    {
      var token = tokenIterator.next();
      string += "(" + token.getKey() + ", " + token.getValue() + ")";
    }
    return string;
  }

});
